;-------------------------------
; Space Invaders Test ROM
; Documented and transfered to TASM
; By Timothy Shiels
; 08/2008
; www.outerworldarcade.com
;-------------------------------
			.area TEST_RAM (REL)

			.globl endramtest
;---------------------------------------
; First RAM test
;---------------------------------------
ramtest::   mvi     c,0x03       ; We'll do the ram tests three times - keep track in c

            ;---------------------------------------            
            ;Store #0x55 in all memory 0x2000 - 0x4000             
            ;---------------------------------------                        
ramtest1:   mvi     b,0x55
            lxi     h,0x2000
rmtst1:		mov     m,b 
            inx     h
            mov     a,h
            cpi     0x40
            jnz     rmtst1
            
            ;---------------------------------------             
            ;Check each stored value 0x2000 - 0x4000         
            ;---------------------------------------       
            lxi     h,0x2000                                       
rmtst2:		mov     a,m                                         
            xra     b                                              
            jnz     badram          ; jump to 0x0190 if not what we expect
            inx     h                                             
            mov     a,h                                            
            cpi     0x40                                            
            jnz     rmtst2                                       
            
            ;---------------------------------------       
            ; Store #0xAA in all memory 0x2000 - 0x4000       
            ;---------------------------------------       
            mvi     b,0xaa                                               
            lxi     h,0x2000                                       
rmtst3:		mov     m,b                                         
            inx     h                                             
            mov     a,h                                            
            cpi     0x40                                            
            jnz     rmtst3
            
            ;---------------------------------------                               
            ;Check each stored value 0x2000 - 0x4000                                 
            ;---------------------------------------                                           
            lxi     h,0x2000                                       
rmtst4:		mov     a,m                                                                               
            xra     b                                              
            jnz     badram      ; jump to 0x0190 if not what we expect 
            inx     h                                             
            mov     a,h                                            
            cpi     0x40                                            
            jnz     rmtst4                                       
                                                                   
            dcr     c                                                      
            jnz     ramtest1    ; start again if not done with 3 tests

;----------------------------------------------
; Second RAM test
; fill memory with incremental values 0x00 - 0xFF
;----------------------------------------------     
ramtest2:   lxi     h,0x2000
            mvi     b,0x00
rmtst5:	    mov     a,b
            mov     m,a
            inx     h
            inr     b
            mov     a,b
            cpi     0xff
            jnz     rmtst6
            mvi     b,0x00       ;start back at 0x00
rmtst6:   	mov     a,h
            cpi     0x40
            jnz     rmtst5

            ;---------------------
            ; verify each one
            ;---------------------
            lxi     h,0x2000
            mvi     b,0x00
rmtst7:    	mov     a,m
            xra     b
            jnz     badram      ; jump to 0x190 if not what we were expecting
            inr     b
            mov     a,b
            cpi     0xff
            jnz     rmtst8
            mvi     b,0x00
rmtst8:    	inx     h
            mov     a,h
            cpi     0x40
            jnz     rmtst7         
            jmp     endramtest    

;--------------
;Bad RAM found
;--------------

badram:     mvi     b,0x01       ;B = counter for where error exists in byte
bdlp:       rrc                 ;rotate byte right
            jc      skip       ;did we find an "ON" bit?
            inr     b           ;NO- increment counter
            jmp     bdlp       ;try again
skip:       mov     a,l         ;YES- load L into A
            lxi     h,charnum-8   ;starting address for images of number error codes
            rrc
            jnc     _num        ;if bad memory location address is even number stay with numbers
            lxi     h,charbad-8 ;starting address for images of letter error codes
_num:       mov     a,b         ;load result from counter where error exists 
            rlc
            rlc
            rlc
            mov     b,a
            mov     e,b
            mvi     d,0x00
            dad     d
            xchg                ;DE= character location pointer
            mvi     b,0x08       ;B= 8 lines
            lxi     h,0x3011     ;HL = location to draw to the screen
_draw:      ldax    d           ;load A with a byte of character image
            mov     m,a         ;draw a line of error code image to screen
            mov     a,b         ;save B
            lxi     b,0x0020     ;next screen location is +0x20
            dad     b           ;add +0x20 to HL
            mov     b,a         ;restore B
            inx     d           ;increment character location pointer
            dcr     b
            jnz     _draw       ;continue to next line if B isn't zero
            mvi     b,0x08       ;B=8 lines
            lxi     h,0x3012     ;HL = location to draw to the screen
            lxi     d,0x0340     ;DE = location of " " character to draw
            out     0x06
            jmp     _draw       ;keep drawing blank character forever


; CHARACTER IMAGES USED BY BAD RAM ROUTINE
charbad:    .byte 0x20, 0x50, 0x88, 0x88, 0xF8, 0x88, 0x88, 0x00 ;"A"
            .byte 0x78, 0x88, 0x88, 0x78, 0x88, 0x88, 0x78, 0x00 ;"B"
            .byte 0x70, 0x88, 0x08, 0x08, 0x08, 0x88, 0x70, 0x00 ;"C"
            .byte 0x70, 0x90, 0x90, 0x90, 0x90, 0x90, 0x70, 0x00 ;"D"
            .byte 0xE0, 0x20, 0x20, 0xE0, 0x20, 0x20, 0xE0, 0x00 ;"E"
            .byte 0x3E, 0x02, 0x02, 0x1E, 0x02, 0x02, 0x02, 0x00 ;"F"
            .byte 0x3C, 0x02, 0x02, 0x02, 0x32, 0x22, 0x3C, 0x00 ;"G"
            .byte 0x22, 0x22, 0x22, 0x3E, 0x22, 0x22, 0x22, 0x00 ;"H"
          
charnum:    .byte 0x00, 0x20, 0x30, 0x20, 0x20, 0x20, 0x20, 0x70 ;"1" (0x1c)
            .byte 0x00, 0x70, 0x88, 0x80, 0x60, 0x10, 0x08, 0xF8 ;"2" (0x1d)
            .byte 0x00, 0xF8, 0x80, 0x40, 0x60, 0x80, 0x88, 0x70 ;"3" (0x1e)
            .byte 0x00, 0x40, 0x60, 0x50, 0x48, 0xF8, 0x40, 0x40 ;"4" (0x1f)
            .byte 0x00, 0xF8, 0x08, 0x78, 0x80, 0x80, 0x88, 0x70 ;"5" (0x20)
            .byte 0x00, 0xE0, 0x10, 0x08, 0x78, 0x88, 0x88, 0x70 ;"6" (0x21)
            .byte 0x00, 0xF8, 0x80, 0x40, 0x20, 0x10, 0x10, 0x10 ;"7" (0x22)
            .byte 0x00, 0x70, 0x88, 0x88, 0x70, 0x88, 0x88, 0x70 ;"8" (0x23)

                                                        